unit uSmsUtils;

interface

uses
  Classes,
  Types,
  XSBuiltIns,
  Dialogs,
  IdHTTP,
  xmldom,
  XMLIntf,
  msxmldom,
  SysUtils,
  DateUtils,
  StrUtils,
  XMLDoc;

type
  TMessageInfoRec = record
    MessageId: Integer;
    UserId: Integer;
    Tariff: Double;
    Content: string;
    ActionDateTime: TDateTime;
    MessageType: Integer;
    Sender: string;
    Receiver: string;
    IsFlash: Boolean;
    Pages: Integer;
    SmsLang: Integer;
    SmsStatus: Integer;
    Title: string;
    Count: Integer;
    Sent: Integer;
    Description: string;
    NotSent: Integer;
    MoneyIsRefunded: Boolean;
    IsRead: Boolean;
  end;
  ArrayOfMessageInfoRec = array of TMessageInfoRec;
  ArrayOfString = array of string;
  ArrayOfInteger = array of Integer;

  TSendSmsWS = class
  private
    class function GetMessageType(MsgType: string): Integer;
    class function GetSmsLang(SmsLang: string): Integer;
    class function GetSmsStatus(SmsStatus: string): Integer;
  public
    class function GetCredit(const UserName, Password: string): Double;
    class function GetInboxCount(const UserName, Password: string; const IsRead: Boolean): Integer;
    class function GetSenderNumbers(const UserName, Password: string): string;

    class function SendSms(const UserName, Password: string;
      const ToNumers: ArrayOfString; const SenderNumber, MessageText: string;
      const IsFlash: Boolean; var SmsId: Integer): Integer;

    class function GetDelivery(const UserName, Password: string;
      const SmsId: Integer; var Numbers: ArrayOfString;
      var DeliveryStatus: ArrayOfInteger;
      const Index, Count: Integer): Integer;

    class function GetMessages(const UserName, Password: string;
      const MessageType: Integer; const SenderNumbers: ArrayOfString;
      const Index, Count: Integer): ArrayOfMessageInfoRec;
  end;

const
  XMLHeader =
    '<?xml version="1.0" encoding="utf-8"?>' +
    '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">' +
    '<soap:Body>';

  XMLFoolter =
    '</soap:Body>' +
    '</soap:Envelope>';

  SendWebService = 'http://payamak-service.ir/SendService.svc';

  ErrorComunication = '     ';
  ErrorUserIsEmpty = '  ->    ';
  ErrorPasswordIsEmpty = '  ->    ';
  ErrorMessageTextIsEmpty = '  ->    ';
  ErrorToNumersIsEmpty = '  ->    ';

implementation

class function TSendSmsWS.SendSms(const UserName, Password: string;
  const ToNumers: ArrayOfString; const SenderNumber, MessageText: string;
  const IsFlash: Boolean; var SmsId: Integer): Integer;
var
  Msg: WideString;
  Response: TStringStream;
  Request: TStringStream;
  I: Integer;
  XML, Flash: WideString;
  Len: integer;
  HttpClient: TIdHTTP;
  XmlDoc: TXmlDocument;
begin
  if Trim(UserName) = '' then
    raise Exception.Create(ErrorUserIsEmpty);

  if Trim(Password) = '' then
    raise Exception.Create(ErrorPasswordIsEmpty);

  if Trim(MessageText) = '' then
    raise Exception.Create(ErrorMessageTextIsEmpty);

  if Length(ToNumers) = 0 then
    raise Exception.Create(ErrorToNumersIsEmpty);

  Result := -1;
  HttpClient := TIdHTTP.Create(nil);
  XmlDoc := TXmlDocument.Create(nil);
  try
    if isflash then
      flash := 'true'
    else
      flash := 'false';

    Len := Length(ToNumers);
    msg := UTF8Encode(MessageText);

    XML := XMLHeader +
      '<SendBatchSms xmlns="http://tempuri.org/">' +
      '<userName>' + UserName + '</userName>' +
      '<password>' + Password + '</password>' +
      '<fromNumber>' + SenderNumber + '</fromNumber>' +
      '<toNumbers xmlns:a="http://schemas.microsoft.com/2003/10/Serialization/Arrays">';

    for I := 1 to Len do
      XML := XML + '<a:string>' + ToNumers[I - 1] + '</a:string>';

    XML := XML +
      '</toNumbers>' +
      '<messageContent>' + msg + '</messageContent>' +
      '<isFlash>' + flash + '</isFlash>' +
      '<batchSmsId/>' +
      '</SendBatchSms>';

    XML := XML + XMLFoolter;

    Request := TStringStream.Create(XML);
    HttpClient.Request.CustomHeaders.Text := 'SOAPAction: "http://tempuri.org/ISendService/SendBatchSms"';
    HttpClient.Request.ContentType := 'text/xml;charset=UTF-8';
    Response := TStringStream.Create('');
    try
      HttpClient.Post(SendWebService, Request, Response);
      XmlDoc.LoadFromStream(Response);
      SmsId := StrToIntDef(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[1].childNodes[0].nodeValue, -1);
      Result := StrToIntDef(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[0].nodeValue, -1);
    except
      ShowMessage(ErrorComunication);
    end;
  finally
    HttpClient.Free;
    XmlDoc.Free;
  end;
end;

class function TSendSmsWS.GetCredit(const UserName, Password: string): Double;
var
  Response: TStringStream;
  Request: TStringStream;
  HttpClient: TIdHTTP;
  XmlDoc: TXmlDocument;
begin
  Result := 0;
  if Trim(UserName) = '' then
    raise Exception.Create(ErrorUserIsEmpty);

  if Trim(Password) = '' then
    raise Exception.Create(ErrorPasswordIsEmpty);

  HttpClient := TIdHTTP.Create(nil);
  XmlDoc := TXmlDocument.Create(nil);
  try
    Request := TStringStream.Create(XMLHeader +
      '<GetCredit xmlns="http://tempuri.org/">' +
      '<userName>' + UserName + '</userName>' +
      '<password>' + Password + '</password>' +
      '</GetCredit>' +
      XMLFoolter);

    HttpClient.Request.CustomHeaders.Text := 'SOAPAction: "http://tempuri.org/ISendService/GetCredit"';
    HttpClient.Request.ContentType := 'text/xml;charset=UTF-8';
    Response := TStringStream.Create('');
    try
      HttpClient.Post(SendWebService, Request, Response);
      XmlDoc.LoadFromStream(Response);
      Result := StrToFloat(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[0].nodeValue);
    except
      ShowMessage(ErrorComunication);
    end;
  finally
    HttpClient.Free;
    XmlDoc.Free;
  end;
end;

class function TSendSmsWS.GetInboxCount(const UserName, Password: string; const IsRead: Boolean): Integer;
var
  Response: TStringStream;
  Request: TStringStream;
  Read: string;
  HttpClient: TIdHTTP;
  XmlDoc: TXmlDocument;
begin
  Result := -1;

  HttpClient := TIdHTTP.Create(nil);
  XmlDoc := TXmlDocument.Create(nil);
  try
    if IsRead then
      Read := 'true'
    else
      Read := 'false';

    Request := TStringStream.Create(XMLHeader +
      '<GetInboxCount xmlns="http://tempuri.org/">' +
      '<userName>' + UserName + '</userName>' +
      '<password>' + Password + '</password>' +
      '<isRead>' + Read + '</isRead>' +
      '</GetInboxCount>' +
      XMLFoolter);

    HttpClient.Request.CustomHeaders.Text := 'SOAPAction: "http://tempuri.org/ISendService/GetInboxCount"';
    HttpClient.Request.ContentType := 'text/xml;charset=UTF-8';
    Response := TStringStream.Create('');
    try
      HttpClient.Post(SendWebService, Request, Response);
      XmlDoc.LoadFromStream(Response);
      Result := StrToInt(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[0].nodeValue);
    except
      ShowMessage(ErrorComunication);
    end;
  finally
    HttpClient.Free;
    XmlDoc.Free;
  end;
end;

class function TSendSmsWS.GetMessages(const UserName, Password: string;
  const MessageType: Integer; const SenderNumbers: ArrayOfString;
  const Index, Count: Integer): ArrayOfMessageInfoRec;
var
  Response: TStringStream;
  Request: TStringStream;
  I, Len: Integer;
  HttpClient: TIdHTTP;
  XmlDoc: TXmlDocument;
  XmlContent: string;
begin
  HttpClient := TIdHTTP.Create(nil);
  XmlDoc := TXmlDocument.Create(nil);
  try
    XmlContent := XMLHeader +
      '<GetMessages xmlns="http://tempuri.org/">' +
      '<userName>' + UserName + '</userName>' +
      '<password>' + Password + '</password>' +
      '<messageType>' + IntToStr(MessageType) + '</messageType>' +
      '<fromNumbers xmlns:a="http://schemas.microsoft.com/2003/10/Serialization/Arrays">';

    for I := 1 to Length(SenderNumbers) do
      XmlContent := XmlContent + '<a:string>' + SenderNumbers[I - 1] + '</a:string>';

    XmlContent := XmlContent + '</fromNumbers>' +
      '<index>' + IntToStr(Index) + '</index>' +
      '<count>' + IntToStr(Count) + '</count>' +
      '</GetMessages>' +
      XMLFoolter;

    Request := TStringStream.Create(XmlContent);
    HttpClient.Request.CustomHeaders.Text := 'SOAPAction: "http://tempuri.org/ISendService/GetMessages"';
    HttpClient.Request.ContentType := 'text/xml;charset=UTF-8';
    Response := TStringStream.Create('');
    try
      HttpClient.Post(SendWebService, Request, Response);
      XmlDoc.LoadFromStream(Response);
      Len := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes.length;
      if Len > 0 then
      begin
        SetLength(Result, Len);
        for I := 0 to Len - 1 do
        begin
          Result[I].MessageId := StrToInt(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[7].childNodes[0].nodeValue);
          Result[I].UserId := StrToInt(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[18].childNodes[0].nodeValue);
          Result[I].Tariff := StrToFloat(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[16].childNodes[0].nodeValue);
          Result[I].Content := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[1].childNodes[0].nodeValue;
          Result[I].ActionDateTime := IncMinute(XMLTimeToDateTime(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[0].childNodes[0].nodeValue), -270);
          Result[I].MessageType := GetMessageType(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[8].childNodes[0].nodeValue);
          Result[I].Sender := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[13].childNodes[0].nodeValue;
          Result[I].Receiver := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[12].childNodes[0].nodeValue;
          Result[I].IsFlash := StrToBool(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[4].childNodes[0].nodeValue);
          Result[I].Pages := StrToInt(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[11].childNodes[0].nodeValue);
          Result[I].SmsLang := GetSmsLang(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[6].childNodes[0].nodeValue);
          Result[I].SmsStatus := GetSmsStatus(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[15].childNodes[0].nodeValue);
          Result[I].Title := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[17].childNodes[0].nodeValue;
          Result[I].Count := StrToInt(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[2].childNodes[0].nodeValue);
          Result[I].Sent := StrToInt(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[14].childNodes[0].nodeValue);

          Result[I].Description := '';
          if XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[3].hasChildNodes then
            Result[I].Description := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[3].childNodes[0].nodeValue;

          Result[I].NotSent := StrToInt(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[10].childNodes[0].nodeValue);
          Result[I].MoneyIsRefunded := StrToBool(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[9].childNodes[0].nodeValue);
          Result[I].IsRead := StrToBool(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[I].childNodes[5].childNodes[0].nodeValue);
        end;
      end;
    except
      ShowMessage(ErrorComunication);
    end;
  finally
    HttpClient.Free;
    XmlDoc.Free;
  end;
end;

class function TSendSmsWS.GetDelivery(const UserName, Password: string;
  const SmsId: Integer; var Numbers: ArrayOfString;
  var DeliveryStatus: ArrayOfInteger;
  const Index, Count: Integer): Integer;
var
  Response: TStringStream;
  Request: TStringStream;
  I, Len: Integer;
  XML: WideString;
  HttpClient: TIdHTTP;
  XmlDoc: TXmlDocument;
begin
  if Trim(UserName) = '' then
    raise Exception.Create(ErrorUserIsEmpty);

  if Trim(Password) = '' then
    raise Exception.Create(ErrorPasswordIsEmpty);

  Result := -1;
  HttpClient := TIdHTTP.Create(nil);
  XmlDoc := TXmlDocument.Create(nil);
  try
    XML := XMLHeader +
      '<GetBatchDelivery xmlns="http://tempuri.org/">' +
      '<userName>' + UserName + '</userName>' +
      '<password>' + Password + '</password>' +
      '<batchSmsId>' + IntToStr(SmsId) + '</batchSmsId>' +
      '<numbers/>' +
      '<deliveryStatus/>' +
      '<index>' + IntToStr(Index) + '</index>' +
      '<count>' + IntToStr(Count) + '</count>' +
      '</GetBatchDelivery>';

    XML := XML + XMLFoolter;

    Request := TStringStream.Create(XML);
    HttpClient.Request.CustomHeaders.Text := 'SOAPAction: "http://tempuri.org/ISendService/GetBatchDelivery"';
    HttpClient.Request.ContentType := 'text/xml;charset=UTF-8';
    Response := TStringStream.Create('');
    try
      HttpClient.Post(SendWebService, Request, Response);
      XmlDoc.LoadFromStream(Response);
      Result := StrToIntDef(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[0].nodeValue, -1);

      if Result = 0 then
      begin
        Len := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[1].childNodes.length;
        SetLength(Numbers, Len);
        SetLength(DeliveryStatus, Len);
        for I := 0 to Len - 1 do
        begin
          Numbers[I] := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[1].childNodes[I].childNodes[0].nodeValue;
          DeliveryStatus[I] := StrToIntDef(XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[2].childNodes[I].childNodes[0].nodeValue, -8);
        end;
      end;
    except
      ShowMessage(ErrorComunication);
    end;
  finally
    HttpClient.Free;
    XmlDoc.Free;
  end;
end;

class function TSendSmsWS.GetMessageType(MsgType: string): Integer;
begin
  Result := -10;
  if SameText(MsgType, 'In') then
    Result := 1
  else if SameText(MsgType, 'Out') then
    Result := 2
  else if SameText(MsgType, 'Schedule') then
    Result := 3;
end;

class function TSendSmsWS.GetSmsLang(SmsLang: string): Integer;
begin
  Result := -10;
  if SameText(SmsLang, 'FA') then
    Result := 1
  else if SameText(SmsLang, 'EN') then
    Result := 2;
end;

class function TSendSmsWS.GetSmsStatus(SmsStatus: string): Integer;
begin
  Result := -10;
  if SameText(SmsStatus, 'Pending') then
    Result := 1
  else if SameText(SmsStatus, 'Illegal') then
    Result := 2
  else if SameText(SmsStatus, 'Sending') then
    Result := 3
  else if SameText(SmsStatus, 'NotApproved') then
    Result := 4
  else if SameText(SmsStatus, 'Canceled') then
    Result := 6
  else if SameText(SmsStatus, 'Error') then
    Result := 7
  else if SameText(SmsStatus, 'NoCredit') then
    Result := 8
  else if SameText(SmsStatus, 'NotSent') then
    Result := 9
  else if SameText(SmsStatus, 'WaitForSend') then
    Result := 10;
end;

class function TSendSmsWS.GetSenderNumbers(const UserName,
  Password: string): string;
var
  Response: TStringStream;
  Request: TStringStream;
  HttpClient: TIdHTTP;
  XmlDoc: TXmlDocument;
begin
  Result := '';
  if Trim(UserName) = '' then
    raise Exception.Create(ErrorUserIsEmpty);

  if Trim(Password) = '' then
    raise Exception.Create(ErrorPasswordIsEmpty);

  HttpClient := TIdHTTP.Create(nil);
  XmlDoc := TXmlDocument.Create(nil);
  try
    Request := TStringStream.Create(XMLHeader +
      '<GetSenderNumbers xmlns="http://tempuri.org/">' +
      '<userName>' + UserName + '</userName>' +
      '<password>' + Password + '</password>' +
      '</GetSenderNumbers>' +
      XMLFoolter);

    HttpClient.Request.CustomHeaders.Text := 'SOAPAction: "http://tempuri.org/ISendService/GetSenderNumbers"';
    HttpClient.Request.ContentType := 'text/xml;charset=UTF-8';
    Response := TStringStream.Create('');
    try
      HttpClient.Post(SendWebService, Request, Response);
      XmlDoc.LoadFromStream(Response);
      Result := XmlDoc.DOMDocument.getElementsByTagName('s:Body').item[0].childNodes[0].childNodes[0].childNodes[0].nodeValue;
    except
      ShowMessage(ErrorComunication);
    end;
  finally
    HttpClient.Free;
    XmlDoc.Free;
  end;
end;

end.

